#include <iostream>
#include <string>
#include <chrono>
#include "lib/nBodySim.hpp"

int main(int argc, char** argv) {
    std::string datafile;

    switch (argc) {
        case 1:
            datafile = "../data/data.ascii";
            break;
        case 2:
            datafile = argv[1];
            break;
        default:
            std::cerr << "Usage: " << argv[0] << " [datafile]" << std::endl;
            return 1;
    }

    double dt = 0.01;
    unsigned nSteps = 1000;

    nBodySim sim(datafile);
    unsigned N = sim.getNParticles();
    std::cout << "N = " << N << std::endl;

    /*
    // print positions
    for (unsigned i=0; i<N; ++i) {
        for (unsigned j=0; j<3; ++j) {
            std::cout << sim.getPositions()[3*i+j] << " ";
        }
        std::cout << std::endl;
    }

    // calculate mean interparticle distance
    double meanInterparticleDistance = sim.calculateMeanInterparticleDistance();
    std::cout << "Mean interparticle distance: " << meanInterparticleDistance << std::endl;

    // calculate forces and measure computation time
    auto start = std::chrono::high_resolution_clock::now();
    sim.calculateForces();
    auto stop = std::chrono::high_resolution_clock::now();

    auto duration = std::chrono::duration_cast<std::chrono::microseconds>(stop - start);
    std::cout << "Computation Time: " << duration.count()/1000000.0 << " seconds" << std::endl;

    // calculate mean force magnitude
    double meanForceMagnitude = sim.calculateMeanForceMagnitude();
    std::cout << "Mean force magnitude: " << meanForceMagnitude << std::endl;

    sim.write2file(sim.getForces(), "../out/forces.txt", 3);
    */

    try {
        sim.runSimulation(dt, nSteps);
    } catch (std::exception& e) {
        std::cerr << e.what() << std::endl;
        return 1;
    }

    // save state of simulation
    sim.saveState2file(nSteps, "../out/state.txt");

    return 0;
}